'use strict';

var web3_js = require('@solana/web3.js');
var jsSha3 = require('js-sha3');
var BN = require('bn.js');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var BN__default = /*#__PURE__*/_interopDefault(BN);

// src/compression/cNFT_instruction_args.ts
function getConcurrentMerkleTreeDecoderFactory(maxDepth, maxBufferSize) {
  return web3_js.getStructDecoder([
    ["sequenceNumber", web3_js.getU64Decoder()],
    ["activeIndex", web3_js.getU64Decoder()],
    ["bufferSize", web3_js.getU64Decoder()],
    [
      "changeLogs",
      web3_js.getArrayDecoder(getChangeLogDecoderFactory(maxDepth), {
        size: maxBufferSize
      })
    ],
    ["rightMostPath", getPathDecoderFactory(maxDepth)]
  ]);
}
function getChangeLogDecoderFactory(maxDepth) {
  return web3_js.getStructDecoder([
    ["root", web3_js.getAddressDecoder()],
    ["pathNodes", web3_js.getArrayDecoder(web3_js.getAddressDecoder(), { size: maxDepth })],
    ["index", web3_js.getU32Decoder()],
    ["_padding", web3_js.getU32Decoder()]
  ]);
}
function getPathDecoderFactory(maxDepth) {
  return web3_js.getStructDecoder([
    ["proof", web3_js.getArrayDecoder(web3_js.getAddressDecoder(), { size: maxDepth })],
    ["leaf", web3_js.getAddressDecoder()],
    ["index", web3_js.getU32Decoder()],
    ["_padding", web3_js.getU32Decoder()]
  ]);
}
var CompressionAccountType = /* @__PURE__ */ ((CompressionAccountType2) => {
  CompressionAccountType2[CompressionAccountType2["Uninitialized"] = 0] = "Uninitialized";
  CompressionAccountType2[CompressionAccountType2["ConcurrentMerkleTree"] = 1] = "ConcurrentMerkleTree";
  return CompressionAccountType2;
})(CompressionAccountType || {});
function getConcurrentMerkleTreeHeaderDecoder() {
  return web3_js.getStructDecoder([
    ["accountType", getCompressionAccountTypeDecoder()],
    ["header", getConcurrentMerkleTreeHeaderDataDecoder()]
  ]);
}
function getCompressionAccountTypeDecoder() {
  return web3_js.getEnumDecoder(CompressionAccountType);
}
function getConcurrentMerkleTreeHeaderDataDecoder() {
  return web3_js.getDiscriminatedUnionDecoder([
    ["V1", getConcurrentMerkleTreeHeaderDataV1Decoder()]
  ]);
}
function getConcurrentMerkleTreeHeaderDataV1Decoder() {
  return web3_js.getStructDecoder([
    ["maxBufferSize", web3_js.getU32Decoder()],
    ["maxDepth", web3_js.getU32Decoder()],
    ["authority", web3_js.getAddressDecoder()],
    ["creationSlot", web3_js.getU64Decoder()],
    ["padding", web3_js.getArrayDecoder(web3_js.getU8Decoder(), { size: 6 })]
  ]);
}

// src/compression/merkleTree.ts
async function getCanopyDepth(rpc, merkleTree) {
  const merkleTreeData = await rpc.getAccountInfo(merkleTree, { encoding: "base64" }).send().then((result) => result.value.data[0]);
  const merkleTreeDataBytes = Buffer.from(merkleTreeData, "base64");
  var currOffset = 0;
  const [merkleTreeHeader, headerOffset] = getConcurrentMerkleTreeHeaderDecoder().read(
    merkleTreeDataBytes,
    currOffset
  );
  currOffset = headerOffset;
  const { maxDepth, maxBufferSize } = merkleTreeHeader.header;
  const [, treeOffset] = getConcurrentMerkleTreeDecoderFactory(
    maxDepth,
    maxBufferSize
  ).read(merkleTreeDataBytes, currOffset);
  const canopyDepth = getCanopyDepthFromCanopyByteLength(
    merkleTreeDataBytes.length - treeOffset
  );
  return canopyDepth;
}
function getCanopyDepthFromCanopyByteLength(canopyByteLength) {
  if (canopyByteLength === 0) {
    return 0;
  }
  return Math.log2(canopyByteLength / 32 + 2) - 1;
}
function getTCollectionEncoder() {
  return web3_js.getStructEncoder([
    ["verified", web3_js.getBooleanEncoder()],
    ["key", web3_js.getAddressEncoder()]
  ]);
}
function getTCollectionDecoder() {
  return web3_js.getStructDecoder([
    ["verified", web3_js.getBooleanDecoder()],
    ["key", web3_js.getAddressDecoder()]
  ]);
}
var TTokenProgramVersion = /* @__PURE__ */ ((TTokenProgramVersion2) => {
  TTokenProgramVersion2[TTokenProgramVersion2["Original"] = 0] = "Original";
  TTokenProgramVersion2[TTokenProgramVersion2["Token2022"] = 1] = "Token2022";
  return TTokenProgramVersion2;
})(TTokenProgramVersion || {});
function getTTokenProgramVersionEncoder() {
  return web3_js.getEnumEncoder(TTokenProgramVersion);
}
var TTokenStandard = /* @__PURE__ */ ((TTokenStandard2) => {
  TTokenStandard2[TTokenStandard2["NonFungible"] = 0] = "NonFungible";
  TTokenStandard2[TTokenStandard2["FungibleAsset"] = 1] = "FungibleAsset";
  TTokenStandard2[TTokenStandard2["Fungible"] = 2] = "Fungible";
  TTokenStandard2[TTokenStandard2["NonFungibleEdition"] = 3] = "NonFungibleEdition";
  return TTokenStandard2;
})(TTokenStandard || {});
function getTTokenStandardEncoder() {
  return web3_js.getEnumEncoder(TTokenStandard);
}
var TUseMethod = /* @__PURE__ */ ((TUseMethod3) => {
  TUseMethod3[TUseMethod3["Burn"] = 0] = "Burn";
  TUseMethod3[TUseMethod3["Multiple"] = 1] = "Multiple";
  TUseMethod3[TUseMethod3["Single"] = 2] = "Single";
  return TUseMethod3;
})(TUseMethod || {});
function getTUseMethodEncoder() {
  return web3_js.getEnumEncoder(TUseMethod);
}
function getTUseMethodDecoder() {
  return web3_js.getEnumDecoder(TUseMethod);
}
function getTUsesEncoder() {
  return web3_js.getStructEncoder([
    ["useMethod", getTUseMethodEncoder()],
    ["remaining", web3_js.getU64Encoder()],
    ["total", web3_js.getU64Encoder()]
  ]);
}
function getTUsesDecoder() {
  return web3_js.getStructDecoder([
    ["useMethod", getTUseMethodDecoder()],
    ["remaining", web3_js.getU64Decoder()],
    ["total", web3_js.getU64Decoder()]
  ]);
}
function getMetadataArgsEncoder() {
  return web3_js.getStructEncoder([
    // adds 4 bytes (U32 equiv) offset containing size before each variable sized utf8 string (name, symbol, uri)
    ["name", web3_js.addEncoderSizePrefix(web3_js.getUtf8Encoder(), web3_js.getU32Encoder())],
    ["symbol", web3_js.addEncoderSizePrefix(web3_js.getUtf8Encoder(), web3_js.getU32Encoder())],
    ["uri", web3_js.addEncoderSizePrefix(web3_js.getUtf8Encoder(), web3_js.getU32Encoder())],
    ["sellerFeeBasisPoints", web3_js.getU16Encoder()],
    ["primarySaleHappened", web3_js.getBooleanEncoder()],
    ["isMutable", web3_js.getBooleanEncoder()],
    ["editionNonce", web3_js.getNullableEncoder(web3_js.getU8Encoder())],
    ["tokenStandard", web3_js.getNullableEncoder(getTTokenStandardEncoder())],
    ["collection", web3_js.getNullableEncoder(getTCollectionEncoder())],
    ["uses", web3_js.getNullableEncoder(getTUsesEncoder())],
    ["tokenProgramVersion", getTTokenProgramVersionEncoder()],
    ["creators", web3_js.getArrayEncoder(getTCreatorEncoder())]
  ]);
}
function getTCreatorEncoder() {
  return web3_js.getStructEncoder([
    ["address", web3_js.getBase58Encoder()],
    ["verified", web3_js.getBooleanEncoder()],
    ["share", web3_js.getU8Encoder()]
  ]);
}
function findMetadataArgs(assetFields) {
  if (!assetFields.compression) {
    throw new Error(`${assetFields.id} is not compressed.`);
  }
  const sellerFeeBasisPointsBuffer = assetFields.royalty ? new BN__default.default(assetFields.royalty.basis_points).toBuffer("le", 2) : Buffer.from([0]);
  const dataHashBuffer = new Uint8Array(
    web3_js.getAddressEncoder().encode(web3_js.address(assetFields.compression.data_hash))
  );
  const metadataArgs = convertAssetFieldsToMetadataArgs(assetFields);
  const originalMetadata = { ...metadataArgs };
  const makeDataHash = (metadataArgs2) => Buffer.from(
    jsSha3.keccak_256.digest(
      Buffer.concat([
        computeMetadataArgsHash(metadataArgs2),
        sellerFeeBasisPointsBuffer
      ])
    )
  );
  var hash = makeDataHash(metadataArgs);
  if (hash.equals(dataHashBuffer)) {
    return metadataArgs;
  }
  metadataArgs.tokenStandard = null;
  hash = makeDataHash(metadataArgs);
  if (hash.equals(dataHashBuffer)) {
    return metadataArgs;
  }
  metadataArgs.name = "";
  metadataArgs.uri = "";
  hash = makeDataHash(metadataArgs);
  if (hash.equals(dataHashBuffer)) {
    return metadataArgs;
  }
  metadataArgs.tokenStandard = 0;
  hash = makeDataHash(metadataArgs);
  if (hash.equals(dataHashBuffer)) {
    return metadataArgs;
  }
  metadataArgs.creators.reverse();
  metadataArgs.name = originalMetadata.name;
  metadataArgs.uri = originalMetadata.uri;
  metadataArgs.tokenStandard = originalMetadata.tokenStandard;
  hash = makeDataHash(metadataArgs);
  if (hash.equals(dataHashBuffer)) {
    return metadataArgs;
  }
  throw new Error("Couldn't find a matching configuration of metadataArgs.");
}
function convertAssetFieldsToMetadataArgs(assetFields) {
  const {
    compression,
    content,
    royalty,
    creators,
    uses,
    grouping,
    supply,
    ownership: { owner, delegate },
    mutable
  } = assetFields;
  const coll = grouping ? grouping.find((group) => group.group_key === "collection")?.group_value : "";
  const tokenStandard = content ? content.metadata.token_standard : "";
  return {
    name: content?.metadata?.name ?? "",
    symbol: content?.metadata?.symbol ?? " ",
    uri: content?.json_uri ?? "",
    sellerFeeBasisPoints: royalty?.basis_points ?? 0,
    creators: creators ?? [],
    primarySaleHappened: royalty?.primary_sale_happened ?? false,
    isMutable: mutable,
    editionNonce: supply?.edition_nonce != null ? supply.edition_nonce : null,
    tokenStandard: tokenStandard === "Fungible" ? 2 /* Fungible */ : tokenStandard === "NonFungibleEdition" ? 3 /* NonFungibleEdition */ : tokenStandard === "FungibleAsset" ? 1 /* FungibleAsset */ : tokenStandard === "NonFungible" ? 0 /* NonFungible */ : null,
    collection: coll ? { key: coll, verified: true } : null,
    uses: uses ? {
      useMethod: uses.use_method === "Burn" ? 0 /* Burn */ : uses.use_method === "Multiple" ? 1 : 2 /* Single */,
      remaining: BigInt(uses.remaining),
      total: BigInt(uses.total)
    } : null,
    // currently always Original for cNFTs
    //TODO: check if retrievable from assetFields
    tokenProgramVersion: 0 /* Original */
  };
}
function computeMetadataArgsHash(metadataArgs) {
  const serializedMetadataArgs = getMetadataArgsEncoder().encode(metadataArgs);
  return Buffer.from(jsSha3.keccak_256.digest(serializedMetadataArgs));
}
function metadataArgsToTMetadataArgsArgs(metadataArgs) {
  const tMetadataArgsArgs = {
    name: metadataArgs.name,
    symbol: metadataArgs.symbol,
    uri: metadataArgs.uri,
    sellerFeeBasisPoints: metadataArgs.sellerFeeBasisPoints,
    primarySaleHappened: metadataArgs.primarySaleHappened,
    isMutable: metadataArgs.isMutable,
    editionNonce: metadataArgs.editionNonce,
    tokenStandard: metadataArgs.tokenStandard,
    collection: metadataArgs.collection,
    uses: metadataArgs.uses,
    tokenProgramVersion: metadataArgs.tokenProgramVersion,
    creatorShares: Buffer.from(
      metadataArgs.creators.map((creator) => creator.share)
    ),
    creatorVerified: metadataArgs.creators.map(
      (creator) => creator.verified
    )
  };
  return tMetadataArgsArgs;
}

// src/metadata/tMetadataArgsArgs.ts
function getTMetadataArgsArgs(assetFields) {
  return metadataArgsToTMetadataArgsArgs(findMetadataArgs(assetFields));
}
var TTokenStandardWithProgrammable = /* @__PURE__ */ ((TTokenStandardWithProgrammable2) => {
  TTokenStandardWithProgrammable2[TTokenStandardWithProgrammable2["NonFungible"] = 0] = "NonFungible";
  TTokenStandardWithProgrammable2[TTokenStandardWithProgrammable2["FungibleAsset"] = 1] = "FungibleAsset";
  TTokenStandardWithProgrammable2[TTokenStandardWithProgrammable2["Fungible"] = 2] = "Fungible";
  TTokenStandardWithProgrammable2[TTokenStandardWithProgrammable2["NonFungibleEdition"] = 3] = "NonFungibleEdition";
  TTokenStandardWithProgrammable2[TTokenStandardWithProgrammable2["ProgrammableNonFungible"] = 4] = "ProgrammableNonFungible";
  return TTokenStandardWithProgrammable2;
})(TTokenStandardWithProgrammable || {});
var Key = /* @__PURE__ */ ((Key2) => {
  Key2[Key2["Uninitialized"] = 0] = "Uninitialized";
  Key2[Key2["EditionV1"] = 1] = "EditionV1";
  Key2[Key2["MasterEditionV1"] = 2] = "MasterEditionV1";
  Key2[Key2["ReservationListV1"] = 3] = "ReservationListV1";
  Key2[Key2["MetadataV1"] = 4] = "MetadataV1";
  Key2[Key2["ReservationListV2"] = 5] = "ReservationListV2";
  Key2[Key2["MasterEditionV2"] = 6] = "MasterEditionV2";
  Key2[Key2["EditionMarker"] = 7] = "EditionMarker";
  Key2[Key2["UseAuthorityRecord"] = 8] = "UseAuthorityRecord";
  Key2[Key2["CollectionAuthorityRecord"] = 9] = "CollectionAuthorityRecord";
  Key2[Key2["TokenOwnedEscrow"] = 10] = "TokenOwnedEscrow";
  Key2[Key2["TokenRecord"] = 11] = "TokenRecord";
  Key2[Key2["MetadataDelegate"] = 12] = "MetadataDelegate";
  Key2[Key2["EditionMarkerV2"] = 13] = "EditionMarkerV2";
  return Key2;
})(Key || {});
function getMetadataDecoder() {
  return web3_js.getStructDecoder([
    ["key", web3_js.getEnumDecoder(Key)],
    ["updateAuthority", web3_js.getAddressDecoder()],
    ["mint", web3_js.getAddressDecoder()],
    ["data", getMetadataDataDecoder()],
    ["primarySaleHappened", web3_js.getBooleanDecoder()],
    ["isMutable", web3_js.getBooleanDecoder()],
    ["editionNonce", web3_js.getNullableDecoder(web3_js.getU8Decoder())],
    [
      "tokenStandard",
      web3_js.getNullableDecoder(web3_js.getEnumDecoder(TTokenStandardWithProgrammable))
    ],
    ["collection", web3_js.getNullableDecoder(getTCollectionDecoder())],
    ["uses", web3_js.getNullableDecoder(getTUsesDecoder())],
    ["collectionDetails", web3_js.getNullableDecoder(getCollectionDetailsDecoder())],
    ["programmableConfig", web3_js.getNullableDecoder(getProgrammableConfigDecoder())]
  ]);
}
function getCollectionDetailsDecoder() {
  return web3_js.getDiscriminatedUnionDecoder([
    ["V1", web3_js.getStructDecoder([["size", web3_js.getU64Decoder()]])],
    ["V2", web3_js.getStructDecoder([["padding", web3_js.getArrayDecoder(web3_js.getU8Decoder())]])]
  ]);
}
function getProgrammableConfigDecoder() {
  return web3_js.getDiscriminatedUnionDecoder([
    ["V1", getProgrammableConfigRecordDecoder()]
  ]);
}
function getProgrammableConfigRecordDecoder() {
  return web3_js.getStructDecoder([
    ["ruleSet", web3_js.getNullableDecoder(web3_js.getAddressDecoder())]
  ]);
}
function getMetadataDataDecoder() {
  return web3_js.getStructDecoder([
    ["name", web3_js.addDecoderSizePrefix(web3_js.getUtf8Decoder(), web3_js.getU32Decoder())],
    ["symbol", web3_js.addDecoderSizePrefix(web3_js.getUtf8Decoder(), web3_js.getU32Decoder())],
    ["uri", web3_js.addDecoderSizePrefix(web3_js.getUtf8Decoder(), web3_js.getU32Decoder())],
    ["sellerFeeBasisPoints", web3_js.getU16Decoder()],
    ["creators", web3_js.getNullableDecoder(web3_js.getArrayDecoder(getTCreatorDecoder()))]
  ]);
}
function getTCreatorDecoder() {
  return web3_js.getStructDecoder([
    ["address", web3_js.getAddressDecoder()],
    ["verified", web3_js.getBooleanDecoder()],
    ["share", web3_js.getU8Decoder()]
  ]);
}

// src/metadata/metadata.ts
async function getRulesetFromMetadataPda(rpc, metadataPda) {
  const decodedMetadataData = await fetchMetadata(rpc, metadataPda);
  return decodedMetadataData.programmableConfig?.ruleSet ?? void 0;
}
async function fetchMetadata(rpc, metadataPda) {
  const metadataData = await rpc.getAccountInfo(metadataPda, { encoding: "base64" }).send().then((result) => result.value.data[0]);
  const metadataDataBytes = Buffer.from(metadataData, "base64");
  return getMetadataDecoder().decode(metadataDataBytes);
}

// src/metadata/metaHash.ts
function getMetaHash(assetFields) {
  return computeMetadataArgsHash(findMetadataArgs(assetFields));
}

// src/compression/cNFT_instruction_args.ts
async function getCNFTArgs(rpc, mint, assetFields, proofFields) {
  if (!assetFields.compression) {
    throw new Error(`${mint} is not compressed.`);
  }
  const metaHash = getMetaHash(assetFields);
  const creatorShares = assetFields.creators ? Uint8Array.from(assetFields.creators.map((c) => c.share)) : new Uint8Array();
  const creatorVerified = assetFields.creators ? assetFields.creators.map((c) => c.verified) : [];
  const sellerFeeBasisPoints = assetFields.royalty ? assetFields.royalty.basis_points : 0;
  const dataHash = new Uint8Array(
    web3_js.getAddressEncoder().encode(web3_js.address(assetFields.compression.data_hash))
  );
  const creatorHash = new Uint8Array(
    web3_js.getAddressEncoder().encode(web3_js.address(assetFields.compression.creator_hash))
  );
  const creators = assetFields.creators?.map((c) => [
    web3_js.address(c.address),
    c.share
  ]);
  const { root, merkleTree, proof, index, tMetadataArgsArgs, canopyDepth } = await getCNFTTransferArgs(rpc, assetFields, proofFields);
  return {
    metaHash,
    creatorShares,
    creatorVerified,
    sellerFeeBasisPoints,
    dataHash,
    creatorHash,
    creators,
    root,
    merkleTree,
    proof,
    index,
    tMetadataArgsArgs,
    canopyDepth
  };
}
async function getCNFTTransferArgs(rpc, assetFields, proofFields) {
  if (!assetFields.compression) {
    throw new Error(`${assetFields.id} is not compressed.`);
  }
  const merkleTree = web3_js.address(proofFields.tree_id);
  const proof = proofFields.proof.map((proof2) => web3_js.address(proof2));
  const root = new Uint8Array(
    web3_js.getAddressEncoder().encode(web3_js.address(proofFields.root))
  );
  const index = assetFields.compression.leaf_id;
  const tMetadataArgsArgs = getTMetadataArgsArgs(assetFields);
  if (!tMetadataArgsArgs) {
    throw new Error("could not retrieve correct tMetadataArgsArgs");
  }
  const canopyDepth = await getCanopyDepth(rpc, merkleTree);
  return { root, merkleTree, proof, index, tMetadataArgsArgs, canopyDepth };
}

// src/DAS/helius_queries.ts
async function retrieveProofFields(helius_url, mint) {
  const requestOptions = {
    method: "POST",
    headers: { "Content-Type": "application/json" },
    body: JSON.stringify({
      jsonrpc: "2.0",
      id: "0",
      method: "getAssetProof",
      params: {
        id: mint
      }
    })
  };
  try {
    const response = await fetch(helius_url, requestOptions);
    if (!response.ok) {
      throw new Error("Failed to fetch data");
    }
    const data = await response.json();
    return data.result;
  } catch (error) {
    throw new Error("Error fetching data: " + error.message);
  }
}
async function retrieveAssetFields(helius_url, mint) {
  const requestOptions = {
    method: "POST",
    headers: { "Content-Type": "application/json" },
    body: JSON.stringify({
      jsonrpc: "2.0",
      id: "0",
      method: "getAsset",
      params: {
        id: mint
      }
    })
  };
  try {
    const response = await fetch(helius_url, requestOptions);
    if (!response.ok) {
      throw new Error("Failed to fetch data");
    }
    const data = await response.json();
    return data.result;
  } catch (error) {
    throw new Error("Error fetching data: " + (error.message || error));
  }
}
async function simulateTxWithIxs(rpc, ixs, signer) {
  const { value: latestBlockhash } = await rpc.getLatestBlockhash().send();
  const simPipe = web3_js.pipe(
    web3_js.createTransactionMessage({ version: 0 }),
    ...ixs.map(
      (ix) => (tx) => web3_js.appendTransactionMessageInstruction(ix, tx)
    ),
    (tx) => web3_js.setTransactionMessageFeePayer(signer.address, tx),
    (tx) => web3_js.setTransactionMessageLifetimeUsingBlockhash(latestBlockhash, tx),
    (tx) => web3_js.compileTransaction(tx),
    (tx) => web3_js.getBase64EncodedWireTransaction(tx)
  );
  const simulationResponse = await rpc.simulateTransaction(simPipe, {
    encoding: "base64",
    sigVerify: false,
    replaceRecentBlockhash: true
  }).send();
  console.log(simulationResponse);
}

exports.fetchMetadata = fetchMetadata;
exports.getCNFTArgs = getCNFTArgs;
exports.getCNFTTransferArgs = getCNFTTransferArgs;
exports.getCanopyDepth = getCanopyDepth;
exports.getMetaHash = getMetaHash;
exports.getRulesetFromMetadataPda = getRulesetFromMetadataPda;
exports.getTMetadataArgsArgs = getTMetadataArgsArgs;
exports.retrieveAssetFields = retrieveAssetFields;
exports.retrieveProofFields = retrieveProofFields;
exports.simulateTxWithIxs = simulateTxWithIxs;
//# sourceMappingURL=index.js.map
//# sourceMappingURL=index.js.map